#!/usr/bin/python3
import os
import tkinter as tk
from tkinter import ttk

from wg_func import (TunnelActiv, ListTunnels, ImportTunnel,
                     ConToDict, GreenLabel, StartStopBTN, ExportTunnels)


class MainWindow(tk.Tk):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.switch_on = None
        self.switch_off = None
        self.x_width = 600
        self.y_height = 350
        self.monitor_center_x = self.winfo_screenwidth() / 2 - (self.x_width / 2)
        self.monitor_center_y = self.winfo_screenheight() / 2 - (self.y_height / 2)
        self.resizable(width=False, height=False)
        self.title('Wire-Py')
        self.configure()
        self.geometry('%dx%d+%d+%d' % (self.x_width, self.y_height, self.monitor_center_x, self.monitor_center_y))
        # Load the image file from disk.
        self.wg_icon = tk.PhotoImage(file=r'icons/wg-vpn-48.png')
        # Set it as the window icon.
        self.iconphoto(True, self.wg_icon)

        FrameWidgets(self).grid()


class FrameWidgets(ttk.Frame):
    def __init__(self, container, **kwargs):
        super().__init__(container, **kwargs)

        self.lb_tunnel = None
        self.wg_read = None
        self.wg_vpn_start = tk.PhotoImage(file=r'icons/wg-vpn-start-48.png')
        self.wg_vpn_stop = tk.PhotoImage(file=r'icons/wg-vpn-stop-48.png')
        self.imp_pic = tk.PhotoImage(file=r'icons/wg-import.png')
        self.tr_pic = tk.PhotoImage(file=r'icons/wg-trash-48.png')
        self.exp_pic = tk.PhotoImage(file=r'icons/wg-export-48.png')
        self.warning_pic = tk.PhotoImage(file=r'icons/warning_64.png')
        # Show active Tunnel
        self.a = TunnelActiv.active()
        # Label 2
        self.lb_frame = ttk.Frame(self)
        self.lb_frame.configure(relief='solid')
        self.lb_frame.grid(column=2, row=2, sticky='n', padx=10)
        self.lb_frame2 = ttk.Frame(self)
        self.lb_frame2.configure(relief='solid')
        self.lb_frame2.grid(column=2, row=3, sticky='nw', padx=10)
        # Show active Label
        self.select_tunnel = None
        self.lb = tk.Label(self, text='Active:')
        self.lb.config(font=("Ubuntu", 11, "bold"))
        self.lb.grid(column=2, row=1, padx=5, sticky="we")

        # Label to Show active Tunnel
        self.StrVar = tk.StringVar(value=self.a)
        GreenLabel.green_show_label(self)

        # Interface Label
        self.interface = tk.Label(self.lb_frame, text='Interface')
        self.interface.grid(column=0, row=4, sticky="n", padx=10)
        self.interface.config(font=("Ubuntu", 9))
        # Address Label
        self.address = tk.Label(self.lb_frame, text='address')
        self.address.grid(column=0, row=5, sticky="n", padx=10)
        self.address.config(font=("Ubuntu", 9))
        # DNS Label
        self.dns = tk.Label(self.lb_frame, text='dns')
        self.dns.grid(column=0, row=6, sticky="n", padx=10)
        self.dns.config(font=("Ubuntu", 9))
        # Peer Label
        self.peer = tk.Label(self.lb_frame2, text='Peer')
        self.peer.config(font=("Ubuntu", 9))
        self.peer.grid(column=0, row=7, sticky="n", padx=10)
        # Endpoint Label
        self.endpoint = tk.Label(self.lb_frame2, text='Endpoint')
        self.endpoint.config(font=("Ubuntu", 9))
        self.endpoint.grid(column=0, row=8, sticky="n", padx=10)

        self.scrollbar = tk.Scrollbar(self)
        self.l_box = tk.Listbox(self, fg='#606060', selectmode='single')
        self.l_box.config(highlightthickness=0, relief='flat')
        self.scrollbar.config(command=self.l_box.yview)
        self.l_box.config(font=("Ubuntu", 12, "bold"))
        self.l_box.grid(column=1, rowspan=3, row=1, )

        self.tl = ListTunnels.tl_list()
        for tunnels in self.tl:
            self.l_box.insert("end", tunnels)
            self.l_box.update()

        # Button Vpn
        if self.a != '':
            StartStopBTN.button_stop(self)
            wg_read = os.environ['HOME'] + '/tester/' + str(self.a) + '.conf'
            file = open(wg_read, 'r')
            ConToDict.covert_to_dict(file)
            file.close()
        else:
            StartStopBTN.button_start(self)
        # Button Import
        self.btn_i = tk.Button(self, image=self.imp_pic, bd=0, command=lambda: ImportTunnel.wg_import_select(self))
        self.btn_i.grid(column=0, row=2, padx=15, pady=15)

        def delete():
            self.select_tunnel = self.l_box.curselection()
            select_tl = self.l_box.get(self.select_tunnel[0])
            os.system('nmcli connection delete ' + str(select_tl))
            self.l_box.delete(self.select_tunnel[0])
            os.remove(os.environ['HOME'] + '/tester/' + str(select_tl) + '.conf')
            if self.a != '':
                self.StrVar.set(value='')
                StartStopBTN.button_start(self)
                self.l_box.update()

        # Button Trash
        self.btn_tr = tk.Button(self, image=self.tr_pic, bd=0, command=delete)
        self.btn_tr.grid(column=0, row=3, padx=15, pady=15)
        # Button Export
        self.btn_exp = tk.Button(self, image=self.exp_pic, bd=0)
        self.btn_exp.grid(column=0, row=4, padx=15, pady=15)

        # Check Buttons
        self.wg_autostart = tk.Checkbutton(self, text='Autoconnect on PC Start')
        self.wg_autostart.grid(column=1, rowspan=3, row=3)
        self.wg_update = tk.Checkbutton(self, text='Search automatically for\nWire-Py updates')
        self.wg_update.grid(column=1, rowspan=3, row=4)

    def wg_switch(self):
        self.a = TunnelActiv.active()
        if self.a == '':
            StartStopBTN.button_start(self)
            self.select_tunnel = self.l_box.curselection()
            select_tl = self.l_box.get(self.select_tunnel[0])
            os.system('nmcli connection up ' + str(select_tl))
            wg_read = os.environ['HOME'] + '/tester/' + str(select_tl) + '.conf'
            file = open(wg_read, 'r')
            ConToDict.covert_to_dict(file)
            file.close()
            # Button Start/Stop
            StartStopBTN.button_stop(self)
            self.a = TunnelActiv.active()
            self.StrVar = tk.StringVar()
            self.StrVar.set(self.a)
            GreenLabel.green_show_label(self)
        elif self.a != '':
            # Button Start/Stop
            StartStopBTN.button_stop(self)
            os.system('nmcli connection down ' + str(self.a))
            # Button Start/Stop
            StartStopBTN.button_start(self)
            self.a = TunnelActiv.active()
            self.StrVar = tk.StringVar()
            self.StrVar.set(value='                               ')
            GreenLabel.green_show_label(self)


if __name__ == '__main__':
    window = MainWindow()
    window.mainloop()
